Pull request management
===

Reviewing
---
When reviewing a pull request, check the following:

- Ensure the pull request has a good, descriptive name.
- **Important:** Check for binary backwardscompatiblity. NB: new optional parameters are not binary compatible.
- **Important:** set the Milestone.
- Add the applicable Labels.  
  Eg.
  - Part: `file-target`, `nlog-configuration` etc
  - Type: `bug`, `enhancement`, `feature`, `performance`. And  `enhancement` is a change without functional impact. Small features are also labeled as `feature`.
  - Tests: `needs unittests`, `has unittests`
  - Status: `waiting for review`, `almost ready`, `ready for merge`
- Set the Assignee. It must indicate who is currently holding the ball.   
  For example, if you intend to review, assign to yourself. If, after the review, some changes need to be made, assign it back to the PR author.


Applying
---
Things to check before applying the PR.

- Check if the comment of the PR has an `fixes ...` comment.
- Check which documentation has to be done. Preferred to fix the documentation just before the merge of the PR>
- Check for related issues and PR's
- Double check binary backwardscompatiblity.
- Add current milestone.

Build Pipeline 
===


For developing: the following platforms will be used:
- net45;net40-client;net35;netstandard1.3;netstandard1.5;netstandard2.0 (see nlog.csproj)

For releasing, the following additional platfoms will be used: sl4;sl5;wp8;monoandroid44;xamarinios10 - see build.ps1

NuGet package management
===


## Create NuGet packages

Use `msbuild /t:Restore,Pack ...`, see build.ps1

## Publish symbols packages

To www.symbolsource.org

```
nuget push NLog\build\bin\release\NuGetPackages\NLog.4.2.0.symbols.nupkg
nuget push NLog\build\bin\release\NuGetPackages\NLog.Extended.4.2.0.symbols.nupkg
```

## Versions

- "BuildLastMajorVersion" should be `major.0.0`. In NLog 4.x - 4.y: 4.0.0
- "AssemblyFileVersion" should be: `major.minor.patch.appVeyorBuildVersion`, eg. 4.2.2.1251 for NLog 4.2.2
- "BuildVersion" should be: `major.minor.patch` where `.patch` is ommited when 0. E.g 4.0, 4.1, 4.1.1, 4.2

Example of correct version numbers in NuGet Package explorer:

![image](https://cloud.githubusercontent.com/assets/5808377/11546997/fbfad58a-9950-11e5-952d-f7369f747089.png)

## XSD file
The XSD file of NLog.Schema is partly generated by "MakeNLogXSD" (inside tools). The enhances the following template: https://github.com/NLog/NLog/blob/master/tools/MakeNLogXSD/TemplateXSD.xml




